//
// ColorTextView.cpp
//

#include "ColorTextView.h"

// Constructor...
CColorTextView::CColorTextView(BRect frame)
	:BTextView(frame, "TextViewColor", BRect(0, 0, 0, 0), B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW)
{	
	SetStylable(true);		// allow multicolor	
	SetWordWrap(false);
	
	// We want a bigger font for the Textview...
	m_pFont = new BFont;
	m_pFont->SetSize(34);
	
	// The user should enter only one line, so the enter character is disallowed...
	DisallowChar('\n');
	
	// Set the default values...
	m_bInputServerWorking = false;
	m_bWrapLine = false;
	m_bUppercaseTags = false;
	m_bQuotationMarks = false;
}

// Every change of the text in the TextView will cause a RefreshHtml().

// InsertText...
void CColorTextView::InsertText(const char* text, int32 length, int32 offset, const text_run_array* runs)
{
	BTextView::InsertText(text, length, offset, runs);
	BRect rectText(0, 0, m_pFont->StringWidth(Text()), 0);
	SetTextRect(rectText);		// change text rect
	Invalidate();				// refresh TextView (Be-Bug?!?)
	RefreshHtml();
}

// DeleteText...
void CColorTextView::DeleteText(int32 start, int32 finish)
{
	BTextView::DeleteText(start, finish);
	BRect rectText(0, 0, m_pFont->StringWidth(Text()), 0);
	SetTextRect(rectText);		// change text rect
	Invalidate();				// refresh TextView
	RefreshHtml();
}

// MessageReceived...
void CColorTextView::MessageReceived(BMessage *message)
{
	if(message->WasDropped())
	{
		this->SomethingDropped(message);
		return;
	}
	
	BTextView::MessageReceived(message);
	switch(message->what)
	{
		// the state of input method has changed
		case B_INPUT_METHOD_EVENT:
		{
			int32 inputMethodEvent;
			message->FindInt32("be:opcode", &inputMethodEvent);
			switch(inputMethodEvent)
			{
				//user starts to input unicode or something
				case B_INPUT_METHOD_STARTED:
					m_bInputServerWorking = true;
					break;
				//user completed to ...
				case B_INPUT_METHOD_STOPPED:
				{
					m_bInputServerWorking = false;
					RefreshHtml();
				}
					break;
			}
		}
			break;
		default :
			break;
	}
}

// RefreshHtml...
void CColorTextView::RefreshHtml()
{
	BString strText = Text();
	m_strHtml = "";			// clear string
	
	// First, we calculate the textlength without spaces.
	// From this value we subtract 1, because the first letter we don't need a color.
	// Special case: When the user just enters only one letter, then the numbers of
	// colors doesn't matter. We must handle this special case, else we would get a
	// "Division by Zero". 
	int nLen = strText.Length();
	int nChar = strText.CountChars();
	int nNumberOfColors = (nChar==1) ? 1 : nChar - 1;
	
	// Now we test if the user entered a text (else: "Division by zero"...)	
	if(nNumberOfColors > 0)
	{
		// Calculate the color steps for each character...
		float fRStep = (m_crEnd.red - m_crStart.red) / nNumberOfColors;		// Red
		float fGStep = (m_crEnd.green - m_crStart.green) / nNumberOfColors;	// Green
		float fBStep = (m_crEnd.blue - m_crStart.blue) / nNumberOfColors;	// Blue

		// We now create the HTML-sourcecode and color the text...
		int nColorNr = 0;		// Number of the color

		BString utf8Char = "";
		for(int i = 0; i <= nLen; i++)	// character for character...	
		{
			if((strText[i] & 0xc0) != 0x80)   //found the next char
			{
				if(utf8Char == " ")		// space?
				{
					// When the flag m_bWrapLine is set, we begin a new line int he
					// HTML-sourcecode after every space-character.
					m_strHtml += m_bWrapLine ? "\n" : " ";
				}
				else if(utf8Char == "") // no char
				{
						
				}			
				else					// normal letter?
				{
					// Calculate the new color...
					rgb_color crNew = {
						m_crStart.red + int(nColorNr * fRStep),			// Red
						m_crStart.green + int(nColorNr * fGStep),		// Green
						m_crStart.blue + int(nColorNr * fBStep),		// Blue
						0};												// Alpha
					nColorNr++;
				
					// Now we add the tags to the source code...
					char strRGB[7]; // string in RGB-Format: "RRGGBB" + "\0" 
					sprintf(strRGB, "%.2x%.2x%.2x", crNew.red, crNew.green, crNew.blue);

					// Uppercase Tags?
					m_strHtml << ((m_bUppercaseTags) ? "<FONT color=" : "<font color=");				
				
					// Use "quotation marks" around the color definition?
					if(m_bQuotationMarks)
					{
					   m_strHtml  << "\"#" << strRGB << "\">";
					}
					else
					{
						m_strHtml << "#" << strRGB << ">";
					}
					
					m_strHtml << utf8Char;		// add character
				
					// Uppercase end-tag?
					m_strHtml << ((m_bUppercaseTags) ? "</FONT>" : "</font>");				
							
					// Set the color in the TextView...
					if(m_bInputServerWorking == false)  //Do not set while input method is working
						SetFontAndColor(i - utf8Char.Length(), i, m_pFont, B_FONT_SIZE, &crNew);				
				}
				utf8Char = strText[i];     //set new char
			}
			//if it's not the top of char, append to new char
			else
			{
				utf8Char << strText[i];
			}
		}
	}

	// Set the text in the Html-TextView...
	m_pTextViewHtml->SetText(m_strHtml.String());
}

// SwapColors...
void CColorTextView::SwapColors()
{
	// Swap the start- and endcolor...
	rgb_color crSwap = m_crStart;
	m_crStart = m_crEnd;
	m_crEnd = crSwap;
	RefreshHtml();
}

//
// *** PRIVATE ROUTINES ***
//

// SomethingDropped... maybe a color ;-)
void CColorTextView::SomethingDropped(BMessage *message)
{
	BPoint dropPoint = message->DropPoint();
	
	type_code type;
	if(message->GetInfo("RGBColor", &type) == B_OK)
	{
		if(type == B_RGB_COLOR_TYPE)
		{
			const rgb_color *color;
			ssize_t numBytes;
			if(message->FindData("RGBColor", B_RGB_COLOR_TYPE, reinterpret_cast<const void **>(&color), &numBytes) == B_OK)
			{
				ConvertFromScreen(&dropPoint);
				float width = this->Bounds().Width();
				if (dropPoint.x < width / 2)
					m_crStart = *color;
				else
					m_crEnd = *color;
				RefreshHtml();
				return;
			}
		}
	}
	BTextView::MessageReceived(message);
}
